"""

Docs

"""

import numpy as np
from theano import tensor


__all__ = ["yield_batches", "create_theano_variable", "tensor_from_array"]


def yield_batches(arrays, batch_size, shuffle=True):
    """
    Yields batches of arrays.
    :type arrays: collections.Sequence[np.ndarray]
    :param arrays: a sequence of arrays to generate batches from, e.g.
                   X-values arrays and Y-values array
    :type batch_size: int
    :param batch_size: the number of entries per batch
    :type shuffle: bool
    :param shuffle: shuffle order (doesn't affect `x_train` and `y_train`
                    objects)
    :rtype: Generator[tuple[np.ndarray]]
    :return: a generator, yielding tuples of array batches
    """
    if len(set(len(array) for array in arrays)) != 1:
        raise ValueError("arrays have different length")

    n_entries = len(arrays[0])

    indices = np.arange(n_entries)
    if shuffle:
        np.random.shuffle(indices)

    return (tuple(array[indices[i:i+batch_size]] for array in arrays)
            for i in range(0, n_entries, batch_size))


def create_theano_variable(ndim, dtype, name=None):
    # TODO docs
    """
    """
    try:
        return {1: tensor.vector(name, dtype=dtype),
                2: tensor.matrix(name, dtype=dtype),
                3: tensor.tensor3(name, dtype=dtype),
                4: tensor.tensor4(name, dtype=dtype)}[ndim]
    except KeyError:
        raise ValueError("`ndim` must be an integer in [1, 4]")


def tensor_from_array(array, name=None):
    # TODO docs
    """
    :type array: np.ndarray
    :param array:
    :type name: str
    :param name:
    :rtype: T.TensorVariable
    """
    return create_theano_variable(ndim=array.ndim,
                                  dtype=str(array.dtype).split(".")[-1],
                                  name=name)


def inject_random_noise(x, p=0.5):
    # Todo docs
    """
    :type x: np.ndarray
    """
    mask = np.random.binomial(1, p, x.size).reshape(x.shape).astype(bool)
    x_hat = x.copy()
    x_hat[mask] = 0
    return x_hat


def inject_gaussian_noise(x, e=0.0, sigma=1.0):
    raise NotImplemented


if __name__ == "__main__":
    raise RuntimeError
